import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.text.*;
import java.io.*;

public class CdDatabase extends JFrame implements ActionListener {

	private JPanel topPanel, bottomPanel, labelPanel, fieldPanel;
	private JButton addButton, updateButton, findButton, deleteButton, exitButton;
	private JLabel codeLabel, titleLabel, priceLabel;
	private JTextField codeField, titleField, priceField;
	private boolean found;
	private int numRecords = 0;
	private CD[] records = new CD[100];
	
	public CdDatabase() {
		setTitle( "CD Database" );
		Toolkit tk = Toolkit.getDefaultToolkit();
		Dimension d = tk.getScreenSize();
		int width = 350, height = 150;
		setBounds( ( d.width - width ) / 2, ( d.height - height ) / 2, width, height );
		setResizable( false );
		Container container = getContentPane();
		
		codeLabel = new JLabel( "Code: " );
		titleLabel = new JLabel( "Title: " );
		priceLabel = new JLabel( "Price: " );
		codeField = new JTextField( "", 20 );
		titleField = new JTextField( "", 20 );
		priceField = new JTextField( "", 20 );
		
		topPanel = new JPanel();
		labelPanel = new JPanel();
		fieldPanel = new JPanel();
		labelPanel.setLayout( new GridLayout( 3, 1 ) );
		fieldPanel.setLayout( new GridLayout( 3, 1 ) );
		
		labelPanel.add( codeLabel );
		labelPanel.add( titleLabel );
		labelPanel.add( priceLabel );
		
		fieldPanel.add( codeField );
		fieldPanel.add( titleField );
		fieldPanel.add( priceField );
		
		topPanel.add( labelPanel );
		topPanel.add( fieldPanel );
		container.add( topPanel, BorderLayout.NORTH );
		
		addButton = new JButton( "Add" );
		updateButton = new JButton( "Update" );
		findButton = new JButton( "Find" );
		deleteButton = new JButton( "Delete" );
		exitButton = new JButton( "Exit" );
		bottomPanel = new JPanel();
		
		bottomPanel.add( addButton );
		bottomPanel.add( updateButton );
		bottomPanel.add( findButton );
		bottomPanel.add( deleteButton );
		bottomPanel.add( exitButton );
		container.add( bottomPanel, BorderLayout.SOUTH );
		
		codeField.addActionListener( this );
		titleField.addActionListener( this );
		priceField.addActionListener( this );
		addButton.addActionListener( this );
		updateButton.addActionListener( this );
		findButton.addActionListener( this );
		deleteButton.addActionListener( this );
		exitButton.addActionListener( this );
		
		enableButtons( true );
	}
	
	public static void main( String[] args ) {
		CdDatabase cdDatabase = new CdDatabase();
		cdDatabase.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		cdDatabase.setVisible( true );
	}
	
	public void actionPerformed( ActionEvent e ) {
		Object source = e.getSource();
		
		if ( source == addButton ) {
			enableButtons( false );
			clearTextFields();
			codeField.requestFocusInWindow();
		}
		else if ( source == updateButton ) {
			if ( numRecords < records.length ) {
				String code = codeField.getText();
				if ( code.length() == 0 ) {
					JOptionPane.showMessageDialog( null, "Code field is empty,\n press Add button again" );
				}
					
				String title = titleField.getText();
				if ( title.length() == 0 ) {
					JOptionPane.showMessageDialog( null, "Title field is empty,\n press Add button again" );
				}
					
				String priceString = priceField.getText();
				boolean digitFlag = validatePrice( priceString );
				if ( digitFlag ) {
					double price = Double.parseDouble( priceString );
					records[numRecords] = new CD( code, title, price );
					numRecords++;
					enableButtons( true );
				} else {
					JOptionPane.showMessageDialog( null,
						"Price field should not contain characters or empty,\n" +
						"please input again" );
					priceField.requestFocusInWindow();
				}
			} else {
				JOptionPane.showMessageDialog( null, "Database is full" );
			}
		}
		else if ( source == findButton ) {
			String findCode = JOptionPane.showInputDialog( "Enter CD code to look up: " );
			found = false;
			for ( int i = 0; i < numRecords; i++ ) {
				String code = records[i].getCode();
				if ( code.equals( findCode ) ) {
					setTextFields( records[i] );
					found = true;
					break;
				}
			}
			if ( ! found ) {
				JOptionPane.showMessageDialog( null, "CD " + findCode + " not found." );
			}
		}
		else if ( source == deleteButton ) {
			String delCode = JOptionPane.showInputDialog( "Enter CD code to delete: " );
			found = false;
			for ( int i = 0; i < numRecords; i++ ) {
				String code = records[i].getCode();
				if ( code.equals( delCode ) ) {
					JOptionPane.showMessageDialog( null, "CD " + delCode + " deleted." );
					records[i] = records[numRecords - 1];
					records[numRecords - 1] = null;
					numRecords--;
					found = true;
						
					if ( records[0] != null ) {
						setTextFields( records[0] );
					} else {
						clearTextFields();
					}
						
					break;
				}
			}
			if ( ! found ) {
				JOptionPane.showMessageDialog( null, "CD " + delCode + " not found." );
			}
		}
		else if ( source == exitButton ) {
			System.exit( 0 );
		}
	}
		
	private void enableButtons( boolean flag ) {
		updateButton.setEnabled( ! flag );
		addButton.setEnabled( flag );
		findButton.setEnabled( flag );
		deleteButton.setEnabled( flag );
	}
		
	private void clearTextFields() {
		codeField.setText( "" );
		titleField.setText( "" );
		priceField.setText( "" );
	}
		
	private void setTextFields( CD cd ) {
		codeField.setText( cd.getCode() );
		titleField.setText( cd.getTitle() );
		priceField.setText( String.valueOf( cd.getPrice() ) );
	}
		
	private boolean validatePrice( String priceString ) {
		int len = priceString.length();
		
		if ( priceString.length() == 0 ) {
			return false;
		}
		for ( int i = 0; i < len; i++ ) {
			char ch = priceString.charAt(i);
			if ( Character.isLetter( ch ) ) {
				return false;
			}
		}
		return true;
	}
}
